@@ -1,41 +1,45 @@
 package App::Virtualenv;
 =head1 NAME
 
-App::Virtualenv - Perl virtual environment
+App::Virtualenv
 
 =head1 VERSION
 
-version 1.07.01
+version 1.03
 
 =head1 SYNOPSIS
 
-Perl virtual environment
+Perl 5 virtual environment module.
 
 =cut
 use strict;
 use warnings;
-no warnings qw(qw utf8);
+no warnings qw(qw utf8 redefine);
 use v5.10;
 use utf8;
-use Config;
 use FindBin;
 use Cwd;
 use File::Basename;
-
-use App::Virtualenv::Utils;
+use Perl::Shell;
+use Term::ReadLine;
+use Config;
+use Lexical::Persistence;
 
 
 BEGIN
 {
 	require Exporter;
 	# set the version for version checking
-	our $VERSION     = '1.07.01';
+	our $VERSION     = '1.03';
 	# Inherit from Exporter to export functions and variables
 	our @ISA         = qw(Exporter);
 	# Functions and variables which are exported by default
 	our @EXPORT      = qw();
 	# Functions and variables which can be optionally exported
 	our @EXPORT_OK   = qw();
+
+	$ENV{PERL_RL} = 'gnu o=0';
+	require Term::ReadLine;
 }
 
 
@@ -45,7 +49,7 @@ sub activate
 	$virtualenvPath = getVirtualenvPath() if not defined $virtualenvPath;
 	$virtualenvPath = binVirtualenvPath() if not defined $virtualenvPath;
 	$virtualenvPath = Cwd::realpath($virtualenvPath);
-	return if not validVirtualenvPath($virtualenvPath);
+	warn "Virtual environment is not valid" if not validVirtualenvPath($virtualenvPath);
 
 	deactivate(1);
 
@@ -123,9 +127,8 @@ sub validVirtualenvPath
 
 sub create
 {
-	my ($virtualenvPath, $empty) = @_;
+	my ($virtualenvPath) = @_;
 	$virtualenvPath = Cwd::realpath((defined $virtualenvPath)? $virtualenvPath: ".");
-	say "Creating Perl virtual environment: $virtualenvPath";
 
 	deactivate();
 
@@ -134,29 +137,105 @@ sub create
 
 	activate($virtualenvPath);
 
-	perl("-MApp::Virtualenv::Module", "-e exit not App::Virtualenv::Module::install(force => 1, test => 0, modules => ['LWP', 'CPAN', 'CPANPLUS']);") unless $empty;
+	system("/usr/bin/perl -MCPAN -e \"CPAN::install('LWP', 'CPAN', 'App::cpanminus', 'App::cpanoutdated')\"");
 
 	my $pkgPath = dirname(__FILE__);
-	_system("cp -v $pkgPath/Virtualenv/activate $virtualenvPath/bin/activate && chmod 644 $virtualenvPath/bin/activate");
-	_system("cp -v $pkgPath/Virtualenv/sh.pl $virtualenvPath/bin/sh.pl && chmod 755 $virtualenvPath/bin/sh.pl");
-	_system("cp -v $pkgPath/Virtualenv/perl.pl $virtualenvPath/bin/perl.pl && chmod 755 $virtualenvPath/bin/perl.pl");
-	_system("ln -v -s -f perl.pl $virtualenvPath/bin/perl");
-	_system("cp -v $pkgPath/Virtualenv/piv.pl $virtualenvPath/bin/piv.pl && chmod 755 $virtualenvPath/bin/piv.pl");
-	_system("ln -v -s -f piv.pl $virtualenvPath/bin/piv");
+	system("cp -v $pkgPath/Virtualenv/activate $virtualenvPath/bin/activate && chmod 644 $virtualenvPath/bin/activate");
+	system("cp -v $pkgPath/Virtualenv/sh.pl $virtualenvPath/bin/sh.pl && chmod 755 $virtualenvPath/bin/sh.pl");
+	system("cp -v $pkgPath/Virtualenv/perl.pl $virtualenvPath/bin/perl.pl && chmod 755 $virtualenvPath/bin/perl.pl");
+	system("cp -v $pkgPath/Virtualenv/shell.pl $virtualenvPath/bin/shell.pl && chmod 755 $virtualenvPath/bin/shell.pl");
 
 	return 1;
 }
 
-sub sh
+sub _system
+{
+	system(@_);
+	if ($? == -1)
+	{
+		warn "failed to execute: $!";
+		return 255;
+	} elsif ($? & 127)
+	{
+		warn "child died with signal ".($? & 127).", ".(($? & 128)? "with": "without")." coredump";
+		return 255;
+	}
+	return $? >> 8;
+}
+
+sub _sh
 {
 	my (@args) = @_;
 	return _system((defined $ENV{SHELL})? $ENV{SHELL}: "/bin/sh", @args);
 }
 
-sub perl
+sub sh
+{
+	my ($virtualenvPath, @args) = @_;
+	$virtualenvPath = activate($virtualenvPath);
+	return _sh(@args);
+}
+
+sub _perl
 {
 	my (@args) = @_;
-	return _system($Config{perlpath}, @args);
+	return _system("/usr/bin/perl", @args);
+}
+
+sub perl
+{
+	my ($virtualenvPath, @args) = @_;
+	$virtualenvPath = activate($virtualenvPath);
+	return _perl(@args);
+}
+
+sub bashReadLine
+{
+	my ($prompt) = @_;
+	$prompt =~ s/\\/\\\\/g;
+	$prompt =~ s/"/\\"/g;
+	$prompt =~ s/\\/\\\\/g;
+	$prompt =~ s/"/\\"/g;
+	my $cmd = '/bin/bash -c "read -p \"'.$prompt.'\" -r -e && echo \"\$REPLY\""';
+	$_ = `$cmd`;
+	chomp;
+	return (not $?)? $_: undef;
+}
+
+#sub Perl::Shell::_readline
+sub _readline
+{
+	my $prompt = shift;
+	if ( -t STDIN ) {
+		return bashReadLine($prompt);
+	} else {
+		print $prompt;
+		my $line = <>;
+		chomp $line if defined $line;
+		return $line;
+	}
+}
+
+sub Perl::Shell::_State::do
+{
+	my $self = shift;
+	$_[0] = "use v$Config{version};\n".$_[0] if defined $_[0];
+	return Lexical::Persistence::do($self, @_);
+}
+
+sub _shell
+{
+	open my $stdout, ">&:encoding(iso-8859-1)", *STDOUT;
+	local *STDOUT = *$stdout;
+	#open my $stdin, "<&:encoding(iso-8859-1)", *STDIN;
+	#local *STDIN = *$stdin;
+	return Perl::Shell::shell();
+}
+
+sub shell
+{
+	my ($virtualenvPath, @args) = @_;
+	return perl($virtualenvPath, "-MApp::Virtualenv", "-eApp::Virtualenv::_shell();");
 }
 
 
